/*!
  @header FTControlCmdLine
  @abstract Main file of FTControlCmdLine tool
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  20.08.06 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/
#if !defined(__FTControlCmdLine)
#define __FTControlCmdLine

#include <Foundation/Foundation.h>
#include <Encore/Encore.h>
#include <FT/FTObject.h>
#include <FTAbstractTool/FTTool.h>
#include <FTAbstractTool/FTAbstractTool.h>


#define FTCONTROLCMDLINE_VERSION @"0.1.0"


/*!
 * @class FTControlCmdLine
 * @abstract Class offering utility methods for the graph database
 */
@interface FTControlCmdLine : FTAbstractTool {
  NSString *login;
  NSString *passwd;
  BOOL cmdListGraphIds;
  BOOL cmdRemoveGraph;
  
  ECStandardOutput *output;
  NSString *graphIdToRemove;
}


/*!
 * @method log
 * @result loggin instance for this application
 */
+ (ECLogger *) log;


/*!
 * @method cmdLineArgumentsOK
 * @result YES if the check for the command line arguments has successfully been
 *   passed
 */
- (BOOL) cmdLineArgumentsOK;


/*!
 * @method cmdLineArgumentsOverview
 * @abstract Prints information on the usage of command line arguments on the 
 *   standard output
 * @result self
 */
- cmdLineArgumentsOverview;


/*!
 * @method cmdListAllGraphIds
 * @abstract this command lists all available graph ids
 * @result self
 */
- cmdListAllGraphIds;


/*!
 * @method cmdRemoveGraph
 * @abstract removes the specified graph from the database
 * @result self
 */
- cmdRemoveGraph;


/*!
 * @method log
 * @result logging instance for FTExport
 */
- (ECLogger *) log;


/*!
 * @method run
 * @abstract runs the export
 * @result self
 */
- run;

@end

#endif
