/*!
  @header FTReferenceImpl
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  30.08.05 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTReferenceImpl_H)
#define __FTReferenceImpl_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <FT/FTReference.h>


/*!
 * @class FTReferenceImpl
 * @abstract Used to store a reference from a node to another node.
 * @discussion In constrast to edges the implementation of this protocol
 *   simply refers to one node, the target node. This node my be either the
 *   source node (in case of an incoming reference) or the target node (in case
 *   of an outgoing reference). */
@interface FTReferenceImpl : FTObject <FTReference, NSCoding> {
  @private
    id <FTId> nodeId;
    id <FTId> edgeId;
}

/*!
 * @method initWithNodeId
 * @abstract Initializer
 * @param aNodeId id of referred node
 * @param anEdgeId id of related edge
 * @result self
 */
- initWithNodeId: (id <FTId>) aNodeId edgeId: (id <FTId>) anEdgeId;
- (void) dealloc;


/* Method used for comparison: */
- (BOOL) isEqual: (id) anObject;
- (unsigned) hash;


/* NSCoding protocol: */
- (id) initWithCoder: (NSCoder *) decoder;
- (void) encodeWithCoder: (NSCoder *) encoder;


/*!
 * @method description
 * @result information about the stae of this object
 */
- (NSString *) description;


/*!
 * @method nodeId
 * @result id of referred node
 */
- (id <FTId>) nodeId;


/*!
 * @method edgeId
 * @result id of related edge
 */
- (id <FTId>) edgeId;

@end
#endif
