/*!
  @header FTServiceAvailability
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  21.09.05 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTServiceAvailability_H)
#define __FTServiceAvailability_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <FT/FTGraph.h>
#include <FT/FTNode.h>

/*!
 * @protocol FTServiceAvailability
 * @abstract Gives information about the availability of a service
 */
@protocol FTServiceAvailability <FTObject>

/*!
 * @method availableForGraph
 * @abstract checks wether the service is available for the given graph
 * @result returns YES if the service is available for the specified graph
 */
- (BOOL) availableForGraph: (id <FTGraph>) aGraph;


/*!
 * @method availableForNodesOfGraph
 * @abstract checks wether the service is available for the specified graph
 * @result YES if the service is available for the specified graph
 */
- (BOOL) availableForNodesOfGraph: (id <FTGraph>) aGraph;


/*!
 * @method availableForNode
 * @abstract checks wether the service is available for the specified node
 * @result YES if the service is available for the specified node
 */
- (BOOL) availableForNode: (id <FTNode>) aNode ofGraph: (id <FTGraph>) aGraph;
@end
#endif
