/*!
  @header FTTransactionContext
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  12.05.05 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTTransactionContext_H)
#define __FTTransactionContext_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>

/*!
 * @class FTTransactionContext
 * @abstract Context the global transaction context which is hand-off to
 *   implementations of {@link FTTransactionStep}
 * @discussion 
 */
@interface FTTransactionContext : FTObject {
  @private
    NSMutableDictionary *contextData;
}

- init;
- (void) dealloc;


/*!
 * @method addObject
 * @abstract Adds an object to the context
 * @throws ECAlreadyExistsException if the key is already been used
 * @result self
 */
- addObject: (id) anObject forKey: (id) keyOfObject;


/*!
 * @method objectForKey
 * @result returns the object associated to the given key or nil if not
 *   existent
 */
- objectForKey: (id) keyOfObject;


/*!
 * @method removeObjectForKey
 * @abstract removes the object associated to the given key. Does nothing if the 
 *   key could not be found
 * @result returns the removed object or nil
 */
- removeObjectForKey: (id) keyOfObject;

@end
#endif
