/*!
  @header FTExport
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  14.02.06 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTExport_H)
#define __FTExport_H

#include <Foundation/Foundation.h>
#include <Encore/Encore.h>
#include <FT/FT.h>
#include <FTAbstractTool/FTTool.h>
#include <FTAbstractTool/FTAbstractTool.h>

#define FTEXPORT_VERSION @"0.1.0"

/*!
 * @class FTExport
 * @abstract Entry class for export utility
 */
@interface FTExport : FTAbstractTool <FTTool> {
  @private
    NSString *graphIdString;

    NSString *exportDir;    
    NSString *baseExportFilename;
    
    NSString *login;
    NSString *passwd;
    
    /* used to write to stdout */
    ECStandardOutput *output;
    
    int fileEncoding;
}


/*!
 * @method log
 * @result logging instance for FTExport
 */
+ (ECLogger *) log;

- init;
- (void) dealloc;


/*!
 * @method cmdLineArgumentsOK
 * @result YES if the check for the command line arguments has successfully been
 *   passed
 */
- (BOOL) cmdLineArgumentsOK;


/*!
 * @method cmdLineArgumentsOverview
 * @abstract Prints information on the usage of command line arguments on the 
 *   standard output
 * @result self
 */
- cmdLineArgumentsOverview;


/*!
 * @method fileEncoding
 * @result the encoding used for the textual export files
 */
- (int) fileEncoding;


/*!
 * @method log
 * @result logging instance for FTExport
 */
- (ECLogger *) log;


/*!
 * @method run
 * @abstract runs the export
 * @result self
 */
- run;

@end
#endif
