/******************************************************************************
  @header FTServer
  @abstract Interface of the server implementation
  
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  31.01.2005 ola     initial version
  23.08.2006 ola     license changed
  -------------------------------------------------------------------------
  </pre>
******************************************************************************/

#if !defined(__FTSERVER_H)
#define __FTSERVER_H

#include <FT/FTObject.h>
#include <FT/FTServiceManager.h>
#include <FT/FTSessionManager.h>
#include <FT/FTConfig.h>


/*!
 * @protocol FTServer
 * @abstract Interface of the FT server
 * @discussionion This is the entry point to get the session manager and the like.
 */
@protocol FTServer <FTObject>

/*!
 * @method config
 * @result the configuration for this server
 * @abstract This configuration is hand-off by the FTBootstrap instance and
 *   read from a ftconfig.xml file.
 */
- (FTConfig *) config;


/*!
 * @method serviceManager
 * @result instance managing the services
 */
- (id <FTServiceManager>) serviceManager;


/*!
 * @method sessionManager
 * @result reference to the session manager
 */
- (id <FTSessionManager>) sessionManager;

@end

#endif



