/*!
  @header FTIdImpl
  @abstract Implementation of the FTId protocol
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  23.02.05 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTIdImpl_H)
#define __FTIdImpl_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <FT/FTId.h>

/*!
 * @class FTIdImpl
 * @abstract Implementation of the FTId protocol
 * @discussionion The current implementation is simply based on strings.
 */
@interface FTIdImpl : FTObject <FTId, NSCoding, NSCopying> {
  @private
    NSString *idString;
}

/*!
 * @method initWithString
 * @abstract initializes this instance with the corresponding id data
 * @result self
 */
- initWithString: (NSString *) aString;


/*!
 * @method initWithCoder
 * @abstract Initializes a newly allocated instance from data in decoder. 
 * @result self
 */
- initWithCoder: (NSCoder *) decoder;
- (void) dealloc;


/*!
 * @method copyWithZone
 * @abstract Returns a new instance that’s a copy of the receiver.
 * @result returns a new instance that’s a copy of the receiver.
 */
- copyWithZone:(NSZone *) zone;


/*!
 * @method description
 * @result information about the content of this object
 */
- (NSString *) description;


/*!
 * @method encodeWithCoder
 * @param encoder instance to be used for reading object's data
 */
- (void) encodeWithCoder: (NSCoder *) encoder;


/*!
 * @method hash
 */
- (unsigned) hash;


/*!
 * @emthod isEqual
 * @abstract compares the receiver with the given object
 * @result YES if both objects do equal, NO otherwise
 */
- (BOOL) isEqual: (id) toCompareWith;
@end
#endif
