/*!
  @header FTAdministrationSession
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  14.03.05 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTAdministrationSession_H)
#define __FTAdministrationSession_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <FT/FTSession.h>

/*!
 * @protocol FTAdministrationSession
 * @abstract Contains administration calls
 */
@protocol FTAdministrationSession <FTObject,FTSession>

/*!
 * @method shutdownServer
 * @abstract kills all sessions except for this one and shutdowns the server
 * @discussion After this call you may only call the method {@link startServer}
 *   of this sessions
 */
- shutdownServer;


/*!
 * @method startServer
 * @abstract starts the server
 * @discussion After this call all databases are mountet and this session is
 *   reconnected to the currently running session manager
 * @result self
 */
- startServer;
@end
#endif
