/*!
  @header FTGraphInfoDBTEST
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  Author: Oliver Langer

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  12.06.06 ola     initial version
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTGraphInfoDBTEST_H)
#define __FTGraphInfoDBTEST_H

#include <Foundation/Foundation.h>
#include <FT/FT.h>
#include "FTBaseTEST.h"


@interface FTGraphInfoDBTEST : FTBaseTEST 
- testInfoDB;
@end
#endif
