/*!
  @header FTDictionaryServiceForNodeImpl
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  17.07.06 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTDictionaryServiceForNodeImpl_H)
#define __FTDictionaryServiceForNodeImpl_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <FT/FTDictionaryService.h>

@class FTDictionaryServiceForGraphImpl;

/*!
 * @class FTDictionaryServiceForNodeImpl
 * @abstract Implements all methods used to use a dictionary service for a
 *  a particular node.
 */
@interface FTDictionaryServiceForNodeImpl : FTObject <FTDictionaryService> {
  @private
    FTDictionaryServiceForGraphImpl *serviceForGraph;
    id <FTNode> node;
}

- initForDictionaryServiceForGraph: 
  (FTDictionaryServiceForGraphImpl *) aServiceForGraph
  forNode: (id <FTNode>) aNode;
- (void) dealloc;


/*!
 * @method allKeys
 * @result all keys
 */
- (id <ECIterator>) allKeys;


/*!
 * @method close
 * @abstract closes this dictionary. After this call the dictionary may not
 *   be used any more
 */
- (void) close;


/*!
 * @method setObject
 * @abstract adds an object to the dictionary. Replaces an existing one if there
 *   is one already defined for the given key
 * @param anObject object to add
 * @param aKey underlying key
 * @result self
 */
- setObject: (id <NSCoding>) anObject forKey: (id <NSCoding>) aKey;


/*!
 * @method objectForKey
 * @abstract returns the object for the given key
 * @param aKey underlying key
 * @result object for specified key
 */
- objectForKey: (id <NSCoding>) aKey;


/*!
 * @method removeObjectForKey
 * @abstract Remove the object associated with the given key. Does nothing if no
 *   object exists for the given key.
 * @param aKey related key
 * @result self
 */
- removeObjectForKey: (id <NSCoding>) aKey;


/*!
 * @method serviceId
 * @result identifier for this services
 */
- (NSString *) serviceId;


/*! 
 * @method version
 * @result return the version of this service
 */
- (id <FTVersion>) serviceVersion;
@end
#endif
