/******************************************************************************
  @header FTSessionManager
  @abstract Used to manage sessions 
 
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  31.01.2005 ola     initial version
  23.08.2006 ola     license changed
  -------------------------------------------------------------------------
  </pre>
******************************************************************************/

#if !defined(__FTSESSIONMANAGER_H)
#define __FTSESSIONMANAGER_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <FT/FTSession.h>

/*!
 * @protocol FTSessionManager
 * @abstract Used to login and for administrative tasks.
 */
@protocol FTSessionManager <FTObject>

/*!
 * @method loginAs
 * @abstract get a session
 * @param loginId 
 * @param password
 * @result FTSession
 */
- (id <FTSession>) loginAs: (NSString *) loginId
  withPassword: (NSString *) password;
  
@end

#endif
