/*!
  @header FTDictionaryServiceTransactionStepImpl
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  17.07.06 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTDictionaryServiceTransactionStepImpl_H)
#define __FTDictionaryServiceTransactionStepImpl_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <FT/FTTransactionStep.h>
#include <FT/FTNode.h>

@class FTDictionaryServiceForGraphImpl;

/*!
 * @class FTDictionaryServiceTransactionStepImpl
 * @abstract A step using to update the dictionary of a particular graph
 */
@interface FTDictionaryServiceTransactionStepImpl 
  : FTObject <FTTransactionStep> 
  

/*!
 * @method transactionForOperationSETOfNode
 * @abstract creates an transaction for adding/updating a dictionary entry
 * @param aNode associated node
 * @param aKey key for entry
 * @param anObject object to set
 * @param serviceImpl underlying service instance
 * @param context transaction context to use
 * @result created transaction step
 */
+ (FTDictionaryServiceTransactionStepImpl *) transactionForOperationSETOfNode: 
  (id <FTNode>) aNode withKey: (id <NSCoding>) aKey 
  withObject: (id <NSCoding>) anObject 
  withDictionaryServiceGraphImpl:(FTDictionaryServiceForGraphImpl *) serviceImpl
   transactionContext: (FTTransactionContext *) context;  


/*!
 * @method transactionForOperationREMOVEOfNode
 * @abstract creates an transaction for removing a dictionary entry
 * @param aNode associated node
 * @param aKey key for entry
 * @param serviceImpl underlying service instance
 * @param context transaction context to use
 * @result created transaction step
 */
+ (FTDictionaryServiceTransactionStepImpl *) transactionForOperationREMOVEOfNode: 
  (id <FTNode>) aNode withKey: (id <NSCoding>) aKey 
   withDictionaryServiceGraphImpl:(FTDictionaryServiceForGraphImpl *) serviceImpl
   transactionContext: (FTTransactionContext *) context;

  
/*!
 * @method performAction 
 * @abstract Called by the transaction controll in order to run the 
 * transaction
 * @result return information about the success of this operation
 */
- (BOOL) performAction: (FTTransactionContext *) transactionContext;


- (BOOL) performActionREMOVEWithContext: (FTTransactionContext *) context;

- (BOOL) performActionSETWithContext: (FTTransactionContext *) context;

/*!
 * @method undoAction
 * @abstract removes all modifications previously been done by 
 *   {@link #performAction}
 * @result return YES if done successfully
 */
- (BOOL) undoAction: (FTTransactionContext *) transactionContext;

@end
#endif
