/*!
  @header FTGraphManager
  @abstract A graph manager is used to find, fetch, create etc. graphs
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  23.02.05 ola     initial version
  19.08.06 ola     added "allGraphIds"
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTGraphManager_H)
#define __FTGraphManager_H

#include <Encore/Encore.h>
#include <FT/FTObject.h>
#include <FT/FTId.h>
#include <FT/FTGraph.h>

/*!
 * @protocol FTGraphManager
 * @abstract Instances of this class are used to find, fetch, load graphs
 */
@protocol FTGraphManager <FTObject>

/*!
 * @method allGraphIds
 * @result an iterator of all graph id's this graph manager manages or
 *   an empty iterator, if no graph exists
 */
- (id <ECIterator>) allGraphIds;


/*! 
 * @method createGraphWithId
 * @abstract create a graph using the given id
 * @param graphId underlying id to use
 * @result created graph
 * @throws ECIllegalArgumentException if a graph with the same identifier 
 *   already exists
 */
- (id <FTGraph>) createGraphWithId: (id <FTId> ) graphId;



/*!
 * @method graphWithId
 * @abstract load the graph with the given id
 * @result the graph with the given id or nil, if it does not exist
 */
- (id <FTGraph>) graphWithId: (id <FTId>) graphId;


/*!
 * @method removeGraphWithId
 * @param graphId id of graph to remove
 * @abstract removes the specified graph or does nothing, if the graph does not
 *   exist
 * @result self
 */
- removeGraphWithId: (id <FTId>) graphId;
@end
#endif
