/*!
  @header FTGraphNodeIteratorImpl
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  10.06.06 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#include <FTGraphNodeIteratorImpl.h>
#include <FT/FTLogging.h>


@implementation FTGraphNodeIteratorImpl

- initForGraph: (FTGraphImpl *) aGraph 
  usingIdIterator: (id <ECIterator>) anIdIterator {
  self = [super init];
  
  self->graph = [aGraph retain];
  self->idIterator = [anIdIterator retain];
  
  return self;
}


- (void) dealloc {
  [self->graph release];
  [self->idIterator release];
  
  [super dealloc];
}


- (BOOL) hasNext {
  return [self->idIterator hasNext];
}


- next {
  id toReturn = nil;
  
  while( [self->idIterator hasNext] && nil == toReturn ) {
    id <FTId> nodeId = [self->idIterator next];
  
    if( nil != nodeId ) {
      toReturn = [self->graph nodeWithId: nodeId];
    } else {
      if( [[FTLogging coreLog] isDebugEnabled] ) {
        [[FTLogging coreLog] 
          debug: @"FTGraphNodeIteratorImpl::next: "\
          "underling node id iterator returns a node id which has no "\
          "corresponding node. Node id=%@. Skipping this node", nodeId];
      }
    }
  }
  
  return toReturn;
}

@end
