/*!
  @header FTImport
  @abstract Main file of import tool
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  27.07.06 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTImport_H)
#define __FTImport_H

#include <Foundation/Foundation.h>
#include <Encore/Encore.h>
#include <FT/FTObject.h>
#include <FTAbstractTool/FTTool.h>
#include <FTAbstractTool/FTAbstractTool.h>
#include "FTImportDefaultXMLRule.h"

#define FTIMPORT_VERSION @"0.1.0"

/*!
 * @class FTImport
 * @abstract Entry class for import utility
 */
@interface FTImport : FTAbstractTool <FTTool> {
  @private
    ECStandardOutput *output;
    NSString *login;
    NSString *passwd;
    
    NSString *targetGraphName;
    NSString *sourceGraphName;
    NSString *sourceBaseDirectory;
    
    FTImportDefaultXMLRule *defaultRule;
}

+ (ECLogger *) log;


- init;
- (void) dealloc;


/*!
 * @method cmdLineArgumentsOK
 * @result YES if the check for the command line arguments has successfully been
 *   passed
 */
- (BOOL) cmdLineArgumentsOK;


/*!
 * @method cmdLineArgumentsOverview
 * @abstract Prints information on the usage of command line arguments on the 
 *   standard output
 * @result self
 */
- cmdLineArgumentsOverview;


/*!
 * @method fileSpecificationsOK
 * @result YES if XML file and the source directory exist
 */
- (BOOL) fileSpecificationsOK;


/*!
 * @method graphDoesNotexist
 * @abstract Returns YES if the specified (new) graph does not exist in the
 *  graph database
 * @result YES if the specified (new) graph does not exist in the
 *  graph database
 */
- (BOOL) graphDoesNotExist;


/*!
 * @method installParsingRules
 * @param control target instance where the rules have to be installed
 * @param aRule rule to install
 * @result self
 */
- installParsingRule: (FTImportDefaultXMLRule *) aRule
  usingXMLControl: (ECXMLControl *) control;


/*!
 * @method log
 * @result logging instance for FTExport
 */
- (ECLogger *) log;


/*!
 * @method run
 * @abstract runs the export
 * @result self
 */
- run;


/*!
 * @method sourceBaseDirectory
 * @result source directory of import
 */
- (NSString *) sourceBaseDirectory;


/*!
 * @method targetGraphName
 * @result name of graph to be created/loaded
 */
- (NSString *) targetGraphName;

@end

#endif
