/******************************************************************************
  FTBaseTest
  FT
 
  Copyright (c) 2004, 2005 oliver langer

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  -------------------------------------------------------------------------
  Modification history

  30.01.2005 ola     initial version
  -------------------------------------------------------------------------
******************************************************************************/

#if !defined(__FTBaseTEST_H) 
#define __FTBaseTEST_H

#include <Foundation/Foundation.h>
#include <Encore/Encore.h>
#include <FT/FT.h>

@interface FTBaseTEST : ECTestcase {
  @protected
    id <FTServer> server;
	id <FTSessionManager> sessionManager;
    id <FTAdministrationSession> session;
	
	BOOL serverStarted;
}
- setup;
- cleanup;

- startServer;
- shutdownServer;
@end

#endif
