/*!
  @header FTGenericTransactionOptimizer
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  24.07.05 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTGenericTransactionOptimizer_H)
#define __FTGenericTransactionOptimizer_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <FT/FTTransactionOptimizer.h>


/*!
 * @class FTGenericTransactionOptimizer
 * @abstract A very simple optimizer which just removes redundancies within 
 *   a set of transaction steps.
 */
@interface FTGenericTransactionOptimizer : FTObject <FTTransactionOptimizer> {
  @private 
    NSLock *lock;
}


- init;
- (void) dealloc;


/*!
 * @method optimizeTransaction
 * @abstract Optimizers must implement this method
 * @discussion Each registered optimizer is called for each 
 *   transaction before this transaction is to be executed (commit or rollback).
 *   Regardless of wether the optimizer ignores the hand-off instance, creates
 *   one, modifies the hand-off: In any case it has to return the (newly) 
 *   transaction instance to be used.
 * @param toOptimize transaction to optimize
 * @result transaction to be used (after optimization)
 */
- (id <FTTransaction>) optimizeTransaction: (id <FTTransaction>) toOptimize;


/*!
 * @method analyseTransaction 
 * @abstract Builds up internal structures in order to optimize the trasnaction
 * @result an internal representation of (parts of) the transaction
 */
- analyseTransaction;
@end
#endif
