/*!
  @header FTGraphNodeIteratorImpl
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  10.06.06 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTGraphNodeIteratorImpl_H)
#define __FTGraphNodeIteratorImpl_H

#include <Foundation/Foundation.h>
#include <Encore/Encore.h>
#include <FT/FTObject.h>
#include <FT/FTGraphImpl.h>


/*!
 * @class FTGraphNodeIteratorImpl
 * @abstract Implements an iterator over all nodes of a graph
 * @discussion Note that this iterator works a bit different from others:
 *   "hasNext" may return YES while in the meanwhile the underlying node
 *   may be removed from the database. So in this case it is possible that
 *   "next" may return nil.
 */
@interface FTGraphNodeIteratorImpl : FTObject <ECIterator> {
  @private
    FTGraphImpl *graph;
    id <ECIterator> idIterator;
}

/*!
 * @method initForGraph
 * @param aGraph graph to examine
 * @param anIdIterator iterator over all node identifiers of this graph
 * @result self
 */
- initForGraph: (FTGraphImpl *) aGraph 
  usingIdIterator: (id <ECIterator>) anIdIterator;
- (void) dealloc;


/*!
 * @method hasNext
 * @result returns YES if there is at least a datum at the current iteration 
 *   step
 */
- (BOOL) hasNext;


/*!
 * @method next
 * @result returns the next object of the set to iterate or nil, if 
 * not existent
 */
- next;
@end
#endif
