/*!
  @header FTLogging
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  13.02.06 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTLogging_H)
#define __FTLogging_H

#include <Foundation/Foundation.h>
#include <FT/FTObject.h>
#include <Encore/Encore.h>

/*!
 * @macro FT_LOGCTX_GENERIC
 * @discussion Generic logging context of FT
 */
#define FT_LOGCTX_GENERIC @"FTGeneric"


/*!
 * @macro FT_LOGCTX_CORE
 * @discussion Logging level of core modules
 */
#define FT_LOGCTX_CORE @"FTCore"


/*!
 * @class FTLogging
 * @abstract Contains all logging instances of FT
 */
@interface FTLogging : FTObject

+ (void) initialize;

/*!
 * @method genericLog
 * @abstract Returns the logging instance for "generic" entries. This
 * may <b>not</b> be used by applications
 * @result logging instance for "generic" entries
 */
+ (ECLogger *) genericLog;


/*!
 * @method coreLog
 * @abstract Returns the logging instance for entries related to the core. This
 * may <b>not</b> be used by applications
 * @result logging instance for entries related to the core
 */
+ (ECLogger *) coreLog;

@end
#endif
