/*!
  @header FTGenericDictionaryProviderImpl
  @abstract Module of FT
 
  @availability OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  01.03.05 ola     initial version
  23.08.06 ola     license changed
  -------------------------------------------------------------------------
  </pre>
*/

#if !defined(__FTGenericDictionaryProviderImpl_H)
#define __FTGenericDictionaryProviderImpl_H

#include <Foundation/Foundation.h>
#include <BDB/BDB.h>
#include <FT/FTObject.h>
#include <FT/FTDictionaryProvider.h>


/*!
 * @class FTGenericDictionaryProviderImpl
 * @abstract A generic implementation of {@link FTDictionaryProvider}
 */
@interface FTGenericDictionaryProviderImpl : FTObject <FTDictionaryProvider> {
  @private
    BDBDatabase *database;
	NSLock *lock;
}


/*!
 * @method initWithDatabase
 * @abstract Initialize this instance with its underlying database
 * @param aDatabase database to use
 * @result self
 */
- initWithDatabase: (BDBDatabase *) aDatabase;
- (void) dealloc;


/*!
 * @method lookupEntryForKey
 * @param aKey key to look for
 * @result return the entry related to the given key or nil if not existent.
 */
- (BDBDatabaseEntry *) lookupEntryForKey: (id <NSCoding>) aKey;


/*!
 * @method objectForKey
 * @param aKey key to look for
 * @result retrieve the object for the specified key or nil if not existent
 */ 
- objectForKey: (id <NSCoding>) aKey;


/*!
 * @method setObject
 * @abstract adds an entry to the dictionary
 * @discussion If an object for the given key already exists then it
 *   will be dropped
 * @param anObject object to put for the given key
 * @param aKey underlying key
 */
- setObject: (id <NSCoding>) anObject forKey: (id <NSCoding>) aKey;

@end
#endif
