/******************************************************************************
  @header FTObject
  @abstract Base class for all FT classes
 
  @availibility OS X, GNUstep
  @copyright 2004, 2005, 2006 Free Software Foundation, Inc.
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  30.01.2005 ola     initial version
  23.08.2006 ola     license changed
  -------------------------------------------------------------------------
  </pre>
******************************************************************************/


#if !defined(__FTOBJECT_H)
#define __FTOBJECT_H

#include <Foundation/Foundation.h>
#include <Encore/Encore.h>

/**
 * @protocol FTObject
 * @abstract interface of all instances derived from FTObject
 */
@protocol FTObject <NSCoding, ECObject>
@end


/**
 * @class FTObject
 * @abstract base interface of all FT objects
 * @discussionion Maybe in future we want to introduce methods being available
 *   for all FT objects.
 */
@interface FTObject :  ECObject <FTObject>

/*!
 * @method initWithCode
 * @abstract initializer called by deserialization process
 * @param decoder to use
 * @result self
 */
- (id) initWithCoder:(NSCoder *) decoder;


/*!
 * @method encodeWithCoder
 * @abstract Is being called in order to serialize this instance
 * @param encoder object to be used for serialization
 */
- (void) encodeWithCoder: (NSCoder *) encoder;

@end

#endif
